@@ -115,10 +115,9 @@ gdm_legacy_display_prepare (GdmDisplay *display)
                                                                             TRUE);
         } else {
                 launch_environment = gdm_create_initial_setup_launch_environment (display_name,
-                                                                                  seat_id,
-                                                                                  NULL,
-                                                                                  NULL,
-                                                                                  TRUE);
+                                                                                seat_id,
+                                                                                NULL,
+                                                                                TRUE);
         }
 
         g_object_set (self, "launch-environment", launch_environment, NULL);
@@ -145,21 +144,6 @@ on_server_ready (GdmServer       *server,
                 g_debug ("GdmDisplay: could not connect to display");
                 gdm_display_unmanage (GDM_DISPLAY (self));
         } else {
-                GdmLaunchEnvironment *launch_environment;
-                char *display_device;
-
-                display_device = gdm_server_get_display_device (server);
-
-                g_object_get (G_OBJECT (self),
-                              "launch-environment", &launch_environment,
-                              NULL);
-                g_object_set (G_OBJECT (launch_environment),
-                              "x11-display-device",
-                              display_device,
-                              NULL);
-                g_clear_pointer(&display_device, g_free);
-                g_clear_object (&launch_environment);
-
                 g_debug ("GdmDisplay: connected to display");
                 g_object_set (G_OBJECT (self), "status", GDM_DISPLAY_MANAGED, NULL);
         }
@@ -263,32 +247,11 @@ gdm_legacy_display_class_init (GdmLegacyDisplayClass *klass)
         g_type_class_add_private (klass, sizeof (GdmLegacyDisplayPrivate));
 }
 
-static void
-on_display_status_changed (GdmLegacyDisplay *self)
-{
-        int status;
-
-        status = gdm_display_get_status (GDM_DISPLAY (self));
-
-        switch (status) {
-            case GDM_DISPLAY_UNMANAGED:
-                if (self->priv->server != NULL)
-                        gdm_server_stop (self->priv->server);
-                break;
-            default:
-                break;
-        }
-}
-
 static void
 gdm_legacy_display_init (GdmLegacyDisplay *legacy_display)
 {
 
         legacy_display->priv = GDM_LEGACY_DISPLAY_GET_PRIVATE (legacy_display);
-
-        g_signal_connect (legacy_display, "notify::status",
-                          G_CALLBACK (on_display_status_changed),
-                          NULL);
 }
 
 GdmDisplay *
