@@ -34,9 +34,11 @@
 #include "gdm-local-display-factory.h"
 #include "gdm-local-display-factory-glue.h"
 
+#include "gdm-settings-keys.h"
+#include "gdm-settings-direct.h"
 #include "gdm-display-store.h"
-#include "gdm-static-display.h"
-#include "gdm-transient-display.h"
+#include "gdm-local-display.h"
+#include "gdm-legacy-display.h"
 
 #define GDM_LOCAL_DISPLAY_FACTORY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_LOCAL_DISPLAY_FACTORY, GdmLocalDisplayFactoryPrivate))
 
@@ -53,7 +55,7 @@ struct GdmLocalDisplayFactoryPrivate
 {
         GdmDBusLocalDisplayFactory *skeleton;
         GDBusConnection *connection;
-        GHashTable      *displays;
+        GHashTable      *used_display_numbers;
 
         /* FIXME: this needs to be per seat? */
         guint            num_failures;
@@ -74,6 +76,7 @@ static void     gdm_local_display_factory_finalize      (GObject
 
 static GdmDisplay *create_display                       (GdmLocalDisplayFactory      *factory,
                                                          const char                  *seat_id,
+                                                         const char                  *session_type,
                                                          gboolean                    initial_display);
 
 static void     on_display_status_changed               (GdmDisplay                  *display,
@@ -132,7 +135,7 @@ take_next_display_number (GdmLocalDisplayFactory *factory)
         ret = 0;
         list = NULL;
 
-        g_hash_table_foreach (factory->priv->displays, (GHFunc)listify_hash, &list);
+        g_hash_table_foreach (factory->priv->used_display_numbers, (GHFunc)listify_hash, &list);
         if (list == NULL) {
                 goto out;
         }
@@ -164,7 +167,7 @@ take_next_display_number (GdmLocalDisplayFactory *factory)
 
         /* now reserve this number */
         g_debug ("GdmLocalDisplayFactory: Reserving X display: %u", ret);
-        g_hash_table_insert (factory->priv->displays, GUINT_TO_POINTER (ret), NULL);
+        g_hash_table_insert (factory->priv->used_display_numbers, GUINT_TO_POINTER (ret), NULL);
 
         return ret;
 }
@@ -178,7 +181,6 @@ on_display_disposed (GdmLocalDisplayFactory *factory,
 
 static void
 store_display (GdmLocalDisplayFactory *factory,
-               guint32                 num,
                GdmDisplay             *display)
 {
         GdmDisplayStore *store;
@@ -190,9 +192,6 @@ store_display (GdmLocalDisplayFactory *factory,
 
         store = gdm_display_factory_get_display_store (GDM_DISPLAY_FACTORY (factory));
         gdm_display_store_add (store, display);
-
-        /* now fill our reserved spot */
-        g_hash_table_insert (factory->priv->displays, GUINT_TO_POINTER (num), NULL);
 }
 
 static const char *
@@ -223,24 +222,36 @@ gdm_local_display_factory_create_transient_display (GdmLocalDisplayFactory *fact
                                                     GError                **error)
 {
         gboolean         ret;
-        GdmDisplay      *display;
-        guint32          num;
+        GdmDisplay      *display = NULL;
         const char      *seat_id;
 
         g_return_val_if_fail (GDM_IS_LOCAL_DISPLAY_FACTORY (factory), FALSE);
 
         ret = FALSE;
 
-        num = take_next_display_number (factory);
+        g_debug ("GdmLocalDisplayFactory: Creating transient display");
 
-        g_debug ("GdmLocalDisplayFactory: Creating transient display %d", num);
+#ifdef WITH_SYSTEMD
+        if (LOGIND_RUNNING() > 0) {
+                display = gdm_local_display_new ();
+        }
+#endif
 
-        display = gdm_transient_display_new (num);
+        if (display == NULL) {
+                guint32 num;
+
+                num = take_next_display_number (factory);
+
+                display = gdm_legacy_display_new (num);
+        }
 
         seat_id = get_seat_of_transient_display (factory);
-        g_object_set (display, "seat-id", seat_id, NULL);
+        g_object_set (display,
+                      "seat-id", seat_id,
+                      "allow-timed-login", FALSE,
+                      NULL);
 
-        store_display (factory, num, display);
+        store_display (factory, display);
 
         if (! gdm_display_manage (display)) {
                 display = NULL;
@@ -269,42 +280,49 @@ on_display_status_changed (GdmDisplay             *display,
         GdmDisplayStore *store;
         int              num;
         char            *seat_id = NULL;
+        char            *session_type = NULL;
         gboolean         is_initial = TRUE;
+        gboolean         is_local = TRUE;
 
         num = -1;
         gdm_display_get_x11_display_number (display, &num, NULL);
-        g_assert (num != -1);
 
         store = gdm_display_factory_get_display_store (GDM_DISPLAY_FACTORY (factory));
 
-        g_object_get (display, "seat-id", &seat_id, NULL);
-        g_object_get (display, "is-initial", &is_initial, NULL);
+        g_object_get (display,
+                      "seat-id", &seat_id,
+                      "is-initial", &is_initial,
+                      "is-local", &is_local,
+                      "session-type", &session_type,
+                      NULL);
 
         status = gdm_display_get_status (display);
 
         g_debug ("GdmLocalDisplayFactory: display status changed: %d", status);
         switch (status) {
         case GDM_DISPLAY_FINISHED:
-                /* remove the display number from factory->priv->displays
+                /* remove the display number from factory->priv->used_display_numbers
                    so that it may be reused */
-                g_hash_table_remove (factory->priv->displays, GUINT_TO_POINTER (num));
+                if (num != -1) {
+                        g_hash_table_remove (factory->priv->used_display_numbers, GUINT_TO_POINTER (num));
+                }
                 gdm_display_store_remove (store, display);
 
                 /* Create a new equivalent display if it was static */
-                if (GDM_IS_STATIC_DISPLAY (display)) {
+                if (is_local) {
                         /* reset num failures */
                         factory->priv->num_failures = 0;
 
-                        create_display (factory, seat_id, is_initial);
+                        create_display (factory, seat_id, session_type, is_initial);
                 }
                 break;
         case GDM_DISPLAY_FAILED:
-                /* leave the display number in factory->priv->displays
+                /* leave the display number in factory->priv->used_display_numbers
                    so that it doesn't get reused */
                 gdm_display_store_remove (store, display);
 
                 /* Create a new equivalent display if it was static */
-                if (GDM_IS_STATIC_DISPLAY (display)) {
+                if (is_local) {
 
                         factory->priv->num_failures++;
 
@@ -314,7 +332,14 @@ on_display_status_changed (GdmDisplay             *display,
                                 /* FIXME: should monitor hardware changes to
                                    try again when seats change */
                         } else {
-                                create_display (factory, seat_id, is_initial);
+#ifdef ENABLE_WAYLAND_SUPPORT
+                                if (g_strcmp0 (session_type, "wayland") == 0) {
+                                        g_free (session_type);
+                                        session_type = NULL;
+                                }
+
+#endif
+                                create_display (factory, seat_id, session_type, is_initial);
                         }
                 }
                 break;
@@ -330,6 +355,7 @@ on_display_status_changed (GdmDisplay             *display,
         }
 
         g_free (seat_id);
+        g_free (session_type);
 }
 
 static gboolean
@@ -353,11 +379,11 @@ lookup_by_seat_id (const char *id,
 static GdmDisplay *
 create_display (GdmLocalDisplayFactory *factory,
                 const char             *seat_id,
+                const char             *session_type,
                 gboolean                initial)
 {
         GdmDisplayStore *store;
-        GdmDisplay      *display;
-        guint32          num;
+        GdmDisplay      *display = NULL;
 
         /* Ensure we don't create the same display more than once */
         store = gdm_display_factory_get_display_store (GDM_DISPLAY_FACTORY (factory));
@@ -368,14 +394,28 @@ create_display (GdmLocalDisplayFactory *factory,
 
         g_debug ("GdmLocalDisplayFactory: Adding display on seat %s", seat_id);
 
-        num = take_next_display_number (factory);
 
-        display = gdm_static_display_new (num);
+#ifdef WITH_SYSTEMD
+        if (g_strcmp0 (seat_id, "seat0") == 0) {
+                display = gdm_local_display_new ();
+                if (session_type != NULL) {
+                        g_object_set (G_OBJECT (display), "session-type", session_type, NULL);
+                }
+        }
+#endif
+
+        if (display == NULL) {
+                guint32 num;
+
+                num = take_next_display_number (factory);
+
+                display = gdm_legacy_display_new (num);
+        }
 
         g_object_set (display, "seat-id", seat_id, NULL);
         g_object_set (display, "is-initial", initial, NULL);
 
-        store_display (factory, num, display);
+        store_display (factory, display);
 
         /* let store own the ref */
         g_object_unref (display);
@@ -395,7 +435,7 @@ delete_display (GdmLocalDisplayFactory *factory,
 
         GdmDisplayStore *store;
 
-        g_debug ("GdmLocalDisplayFactory: Removing displays on seat %s", seat_id);
+        g_debug ("GdmLocalDisplayFactory: Removing used_display_numbers on seat %s", seat_id);
 
         store = gdm_display_factory_get_display_store (GDM_DISPLAY_FACTORY (factory));
         gdm_display_store_foreach_remove (store, lookup_by_seat_id, (gpointer) seat_id);
@@ -429,8 +469,26 @@ static gboolean gdm_local_display_factory_sync_seats (GdmLocalDisplayFactory *fa
         array = g_variant_get_child_value (result, 0);
         g_variant_iter_init (&iter, array);
 
-        while (g_variant_iter_loop (&iter, "(&so)", &seat, NULL))
-                create_display (factory, seat, TRUE);
+        while (g_variant_iter_loop (&iter, "(&so)", &seat, NULL)) {
+                gboolean is_initial;
+                const char *session_type = NULL;
+
+                if (g_strcmp0 (seat, "seat0") == 0) {
+                        is_initial = TRUE;
+#ifdef ENABLE_WAYLAND_SUPPORT
+                        gboolean wayland_enabled = FALSE;
+                        if (gdm_settings_direct_get_boolean (GDM_KEY_INITIAL_SETUP_ENABLE, &wayland_enabled)) {
+                                if (wayland_enabled) {
+                                        session_type = "wayland";
+                                }
+                        }
+#endif
+                } else {
+                        is_initial = FALSE;
+                }
+
+                create_display (factory, seat, session_type, is_initial);
+        }
 
         g_variant_unref (result);
         g_variant_unref (array);
@@ -449,7 +507,7 @@ on_seat_new (GDBusConnection *connection,
         const char *seat;
 
         g_variant_get (parameters, "(&s&o)", &seat, NULL);
-        create_display (GDM_LOCAL_DISPLAY_FACTORY (user_data), seat, FALSE);
+        create_display (GDM_LOCAL_DISPLAY_FACTORY (user_data), seat, NULL, FALSE);
 }
 
 static void
@@ -525,7 +583,7 @@ gdm_local_display_factory_start (GdmDisplayFactory *base_factory)
 #endif
 
         /* On ConsoleKit just create Seat1, and that's it. */
-        display = create_display (factory, CK_SEAT1_PATH, TRUE);
+        display = create_display (factory, CK_SEAT1_PATH, NULL, TRUE);
 
         return display != NULL;
 }
@@ -666,7 +724,7 @@ gdm_local_display_factory_init (GdmLocalDisplayFactory *factory)
 {
         factory->priv = GDM_LOCAL_DISPLAY_FACTORY_GET_PRIVATE (factory);
 
-        factory->priv->displays = g_hash_table_new (NULL, NULL);
+        factory->priv->used_display_numbers = g_hash_table_new (NULL, NULL);
 }
 
 static void
@@ -684,7 +742,7 @@ gdm_local_display_factory_finalize (GObject *object)
         g_clear_object (&factory->priv->connection);
         g_clear_object (&factory->priv->skeleton);
 
-        g_hash_table_destroy (factory->priv->displays);
+        g_hash_table_destroy (factory->priv->used_display_numbers);
 
 #ifdef WITH_SYSTEMD
         gdm_local_display_factory_stop_monitor (factory);
